import {
  select,
  csv,
  scaleLinear,
  max,
  extent,  
  axisLeft,
  axisBottom,
  format
} from 'd3';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

//creates rectangles for each row
// Quantitative attributes
// Domain = Data space
// Range = screen space (pixel coords)

// Quantitative attributes
// Domain = Data space
// Range = screen space (pixel coords)

// mpg,cylinders,displacement,horsepower,weight,acceleration,model_year,origin,name
// d.mpg = +d.mpg;
// d.cylinders = +d.cylinders;
// d.displacement = +d.displacement;
// d.horsepower = +d.horsepower;
// d.weight = +d.weight;
// d.acceleration = +d.acceleration;
// d.model_year = +d.model_year;    


const render = data => {
  
  const xValue = d => d.horsepower;
  const xAxisLabel = "Horsepower";
	const yValue = d => d.weight;
  const yAxisLabel = "Weight";
  const margin = {top:100, right:100, bottom:90, left:150 };
  const innerWidth = width - margin.left - margin.right
  const innerHeight = height - margin.top - margin.bottom
  const circleRadius = 8;
  const title = 'Cars - Horsepower vs weight'
  
  
  const xScale = scaleLinear() // maps domain onto range
  .domain(extent(data, xValue)) //min(data, xValue),max(data, xValue)])
  .range([0, innerWidth]) // min max
  .nice();
  
  const yScale = scaleLinear() // categorical variables
  .domain(extent(data, yValue))
  .range([innerHeight, 0]) // min max
  .nice();
  
  const g = svg.append('g')
  	.attr('transform', `translate(${margin.left},${margin.top})`);
  

  
  const xAxis = axisBottom(xScale)
  	.tickSize(-innerHeight)
  	.tickPadding(15);
 
  const yAxis = axisLeft(yScale)
  .tickSize(-innerWidth)
  .tickPadding(10);
  
  const yAxisG = g
  	.append('g')
    .call(yAxis);
  
  yAxisG
  	.selectAll('.domain')
  	.remove();
  
  yAxisG.append('text')
  	.attr('class', 'axis-label')
    .attr('y', -93)
  	.attr('fill', 'black')
    .attr('x', -innerHeight/2)
    .attr('transform', `rotate(-90)`)
    .attr('text-anchor', 'middle')
    .text(yAxisLabel);


  const xAxisG = g.append('g')
    .call(xAxis)
  	.attr('transform', `translate(0,${innerHeight})`);
  
  xAxisG.select('.domain').remove();
 
  xAxisG.append('text')
  	.attr('class', 'axis-label')
    .attr('y', 75)
  	.attr('fill', 'black')
    .attr('x', innerWidth/2)
    .text(xAxisLabel);

 
  g.selectAll('circle') // currently empty set of rectangles
  .data(data) //the data, bound to empty rectangles. This is the 'd3 data join'
  .enter() // (enter) - creates 1 'slot' for each row
  .append('circle') //append rectangles to your data elements (update)
  	.attr('cy', d => yScale(yValue(d)))
    .attr('cx', d => xScale(xValue(d))) // dimensions of rectangles
    .attr('r', circleRadius);
  
 //add the title
  g.append('text')
    .attr('class', 'title')
    .attr('y', -10)
	  .attr('fill', 'black')
    .text(title);

  
  
};

csv('https://vizhub.com/curran/datasets/auto-mpg.csv')
  .then(data => { // => is the arrow function // then is a promise

    data.forEach(d => { // 1 object per *row* of original table
      d.mpg = +d.mpg;
      d.cylinders = +d.cylinders;
      d.displacement = +d.displacement;
      d.horsepower = +d.horsepower;
      d.weight = +d.weight;
      d.acceleration = +d.acceleration;
      d.model_year = +d.model_year;      
  });
  render(data);
});
